{% extends "base.html" %}

<!--https://squidfunk.github.io/mkdocs-material/customization/#overriding-blocks-->

{% block announce %}
<div style="text-align: center">
  <a href="https://github.com/kyegomez/swarms">Star and contribute</a> to swarms
  on GitHub!
</div>
{% endblock %}
