import os

from swarm_models import OpenAIChat

# Example usage:
api_key = os.getenv("GROQ_API_KEY")

# Model
model = OpenAIChat(
    openai_api_base="https://api.groq.com/openai/v1",
    openai_api_key=api_key,
    model_name="llama-3.1-70b-versatile",
    temperature=0.1,
)


out = model("What is the essence of quantum field theory?")
print(out)
